﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Schema;

namespace XML
{
    public partial class XmlSchemaCollectionSample : Form
    {
        public XmlSchemaCollectionSample()
        {
            InitializeComponent();
        }

        private static string validationErrors = String.Empty;
        private void ValidateBtn_Click(object sender, EventArgs e)
        {

            try
            {
                // looking for the element tags.
                string nodeNames = String.Empty;

                // Read the xml file, and send this to validator.
                XmlTextReader xmlReader = new XmlTextReader(@"ProductWithXsd.xml");

                // load the schema file into an XmlTextReader object
                XmlTextReader xsdReader = new XmlTextReader(@"product.xsd");
                XmlSchemaCollection sc = new XmlSchemaCollection();

                // add the ValidationEventHandler to the schema collection
                sc.ValidationEventHandler += new ValidationEventHandler(ValidationCallBack);

                // add the schema to the XmlSchemaCollection object
                sc.Add("", xsdReader);

                // create an XmlValidatingReader object based on the Xml document
                XmlValidatingReader rdr = new XmlValidatingReader(xmlReader);

                // set the ValidationType to a schema
                rdr.ValidationType = ValidationType.Schema;
                rdr.Schemas.Add(sc);
                rdr.ValidationEventHandler += new ValidationEventHandler(ValidationCallBack);
                while (rdr.Read())
                {
                    if (rdr.NodeType == XmlNodeType.Element)
                        nodeNames += rdr.Name + " ";
                }

                if (validationErrors != String.Empty)
                {
                    // errors were added during the ValidationEventHandler
                    char[] delimiter = new Char[1];
                    delimiter[0] = Convert.ToChar(";");
                    string[] errors = validationErrors.Split(delimiter);
                    for (int i = 0; i < errors.Length-1; i++)
                        MessageBox.Show(errors[i]);
                }
                else
                {
                    // there were no validation errors, so display that message, or use the 'nodeNames'
                    MessageBox.Show("No validation errors.");
                }
            }
            catch (Exception ex)
            {
                validationErrors += ex.Message + ";";
                MessageBox.Show(validationErrors);
            }

        }

        // the ValidationCallBack method is called when a Validation error occurs
        public static void ValidationCallBack(object sender, ValidationEventArgs e)
        {
            validationErrors += e.Message + ";";
        }

        private void XmlSchemaCollectionSample_Load(object sender, EventArgs e)
        {
            xmlFile.Text = Application.StartupPath + "\\ProductWithXsd.xml";
            xsdFile.Text = Application.StartupPath + "\\product.xsd";
        }

    }
}
